@@ -1,8 +1,8 @@
-from typing import Optional
+from typing import Optional, Union, Text
 
 import requests
 
-__version__ = "1.1.1"
+__version__ = "1.2.1"
 READ_METHODS = {"HEAD", "GET", "OPTIONS", "CONNECT", "TRACE"}
 
 
@@ -34,33 +34,30 @@ class APISession(requests.Session):
         # Override this method as needed
         response.raise_for_status()
 
-    def post(self, *args, **kwargs):
-        assert not self.read_only
-        return super().post(*args, **kwargs)
+    def request(self, method: str, url: Union[str, bytes, Text], *args, bypass_read_only=False, **kwargs):
+        """
+        :param method: method argument passed to the underlying ``.request()`` method
+        :param url: URL argument passed to the underlying ``.request()`` method
+        :param args: arguments passed to the underlying ``.request()`` method
+        :param bypass_read_only: if True, ignore the ``.read_only`` attribute
+        :param kwargs: keyword arguments passed to the underlying ``.request()`` method
+        :return:
+        """
+        if self.read_only and not bypass_read_only and method.upper() not in READ_METHODS:
+            raise AssertionError("Can't perform %s action in read-only mode!" % method)
+        return super().request(method, url, *args, **kwargs)
 
-    def put(self, *args, **kwargs):
-        assert not self.read_only
-        return super().put(*args, **kwargs)
-
-    def delete(self, *args, **kwargs):
-        assert not self.read_only
-        return super().delete(*args, **kwargs)
-
-    def request_api(self, method: str, path: str, *args, throw=False, bypass_read_only=False, **kwargs):
+    def request_api(self, method: str, path: str, *args, throw: Optional[bool] = None, **kwargs):
         """
         Wrapper around .request() that prefixes the path with the base API URL.
 
         :param method: HTTP method
         :param path: API path. This must start with "/"
         :param args: arguments passed to ``.request()``
-        :param throw: if True, raise an exception if the response is an error
-        :param bypass_read_only: if True, ignore the ``.read_only`` attribute
+        :param throw: if True, raise an exception if the response is an error.
         :param kwargs: keyword arguments passed to ``.request()``
         :return:
         """
-        if self.read_only and not bypass_read_only and method.upper() not in READ_METHODS:
-            raise RuntimeError("Can't perform %s action in read-only mode!" % method)
-
         assert path.startswith("/")
 
         url = self.base_url + path
@@ -70,7 +67,7 @@ class APISession(requests.Session):
             self.raise_for_response(r)
         return r
 
-    def get_api(self, path: str, params: Optional[dict] = None, *, throw=False, **kwargs):
+    def get_api(self, path: str, params: Optional[dict] = None, *, throw: Optional[bool] = None, **kwargs):
         """
         Equivalent of .get() that prefixes the path with the base API URL.
 
@@ -92,14 +89,14 @@ class APISession(requests.Session):
         :param kwargs: keyword arguments passed to the underlying call
         :return:
         """
-        r = self.get_api(path, params=params, throw=throw, **kwargs)
+        r = self.get_api(path, params=params, throw=False if none_on_404 else throw, **kwargs)
         if none_on_404 and r.status_code == 404:
             return None
         if throw:
             self.raise_for_response(r)
         return r.json()
 
-    def head_api(self, path: str, params: Optional[dict] = None, *, throw=False, **kwargs):
+    def head_api(self, path: str, params: Optional[dict] = None, *, throw: Optional[bool] = None, **kwargs):
         """
         Equivalent of .head() that prefixes the path with the base API URL.
 
@@ -110,7 +107,7 @@ class APISession(requests.Session):
         """
         return self.request_api('head', path, params=params, throw=throw, **kwargs)
 
-    def post_api(self, path: str, *args, throw=False, **kwargs):
+    def post_api(self, path: str, *args, throw: Optional[bool] = None, **kwargs):
         """
         Equivalent of .post() that prefixes the path with the base API URL.
 
@@ -120,7 +117,7 @@ class APISession(requests.Session):
         """
         return self.request_api('post', path, *args, throw=throw, **kwargs)
 
-    def put_api(self, path: str, *args, throw=False, **kwargs):
+    def put_api(self, path: str, *args, throw: Optional[bool] = None, **kwargs):
         """
         Equivalent of .put() that prefixes the path with the base API URL.
 
@@ -130,7 +127,17 @@ class APISession(requests.Session):
         """
         return self.request_api('put', path, *args, throw=throw, **kwargs)
 
-    def delete_api(self, path: str, throw=False, **kwargs):
+    def patch_api(self, path: str, *args, throw: Optional[bool] = None, **kwargs):
+        """
+        Equivalent of .patch() that prefixes the path with the base API URL.
+
+        :param path: URL path. This must start with a slash
+        :param throw: if True, throw an exception on error
+        :return: requests.Response object
+        """
+        return self.request_api('patch', path, *args, throw=throw, **kwargs)
+
+    def delete_api(self, path: str, throw: Optional[bool] = None, **kwargs):
         """
         Equivalent of .delete() that prefixes the path with the base API URL.
 
